import os
import os.path
import sys
import math
from shutil import rmtree
import pytz

from django.db import models, transaction
from django.db.models import Q
from galeria.settings import FORCE_SCRIPT_NAME, STATIC_PATH_PREFIX, \
    UPLOAD_TMP_DIR, LANGUAGE_CODE, TIME_ZONE
from django_comments.moderation import CommentModerator, moderator
from django.core.mail import send_mail

from babel.dates import format_datetime
from datetime import datetime, timezone, timedelta

from fotos.util import get_image_metadata, get_month_name, get_year_name, \
    normalize, free, restrict, getquote, get_view_coords

from galeria.settings import GALERIA_INFO, SYSTEM_EMAIL

class Album(models.Model):

    the_parent = models.ForeignKey('Album', on_delete=models.SET_NULL, null=True)
    name = models.CharField(max_length=60)
    slug = models.CharField(max_length=60)
    first_timestamp = models.DateTimeField()
    last_timestamp = models.DateTimeField()
    time_offset = models.SmallIntegerField(null=True)
    the_photo = models.ForeignKey('Photo', on_delete=models.SET_NULL, null=True)
    private = models.BooleanField(default=False)

    def album_year(self):
        tz = pytz.timezone(TIME_ZONE)
        locale =  LANGUAGE_CODE.replace('-' , '_')
        return format_datetime(self.first_timestamp, 'YYYY', tz, locale)

    def album_month(self):
        tz = pytz.timezone(TIME_ZONE)
        locale =  LANGUAGE_CODE.replace('-' , '_')
        return format_datetime(self.first_timestamp, 'MM', tz, locale)

    def get_url_path(self):
        parts = []
        a = self
        while a:
            parts.insert(0, a.slug)
            a = a.the_parent
        return os.path.join(FORCE_SCRIPT_NAME, *parts)

    def get_day_range(self):
        if self.first_timestamp.day == self.last_timestamp.day:
            return '%d %s' % (self.first_timestamp.day, self.name)
        return '%d-%d %s' % (self.first_timestamp.day,
                             self.last_timestamp.day,
                             self.name)

    def get_subalbums(self, reverse=False):
        order = '-first_timestamp' if reverse else 'first_timestamp'
        return Album.objects.filter(the_parent=self).order_by(order)

    def get_photos(self, reverse=False):
        order = '-timestamp' if reverse else 'timestamp'
        return Photo.objects.filter(the_album=self).order_by(order)

    def get_photo(self, photo):
        return Photo.objects.filter(the_album=self, basename=photo).first()

    def get_subalbum(self, slug):
        return Album.objects \
                    .filter(the_parent=self, slug=slug) \
                    .order_by('first_timestamp').first()

    def get_tree(self, auth=False):
        tree = {}
        subalbums = self.get_subalbums()
        for subalbum in subalbums:
            if not auth and subalbum.private:
                continue
            tree[subalbum.slug] = { 'album': subalbum,
                                    'label': subalbum.get_day_range(),
                                    'tree': subalbum.get_tree() }
        return tree

    def reset(self):
        if not self.the_parent:
            return self._reset_year()
        if not self.the_parent.the_parent:
            return self._reset_month()
        if not self.the_parent.the_parent.the_parent:
            return self._reset_event()
        sys.stderr.write('Invalid album: %(album)' % { 'album': self })

    def get_prev(self):
        if not self.the_parent:
            return self._get_prev_year()
        if not self.the_parent.the_parent:
            return self._get_prev_month()
        if not self.the_parent.the_parent.the_parent:
            return self._get_prev_event()
        sys.stderr.write('Invalid album: %(album)' % { 'album': self })
        return None

    def get_next(self):
        if not self.the_parent:
            return self._get_next_year()
        if not self.the_parent.the_parent:
            return self._get_next_month()
        if not self.the_parent.the_parent.the_parent:
            return self._get_next_event()
        sys.stderr.write('Invalid album: %(album)' % { 'album': self })
        return None

    def get_prev_url(self):
        prev = self.get_prev()
        if prev:
            return prev.get_url_path()
        return None

    def get_next_url(self):
        next = self.get_next()
        if next:
            return next.get_url_path()
        return None

    def set_cover(self):
        if not self.the_parent:
            self._set_cover_year()
        if not self.the_parent.the_parent:
            self._set_cover_month()
        self._set_cover_event()

    def has_public_photos(self):
        photos = self.get_photos()
        if photos:
            for photo in photos:
                if not photo.private:
                    return True
        else:
            for subalbum in self.get_subalbums():
                if subalbum.has_public_photos():
                    return True
        return False

    def make_private(self):
        if self.private:
            return []
        self.private = True
        self.save()
        changed = [ self ]
        photos = self.get_photos()
        if photos:
            for photo in photos:
                if not photo.private:
                    photo.private = True
                    photo.save()
                    changed.append(photo)
        else:
            for subalbum in self.get_subalbums():
                if not subalbum.private:
                    changed += subalbum.make_private()
        if self.the_parent and not self.the_parent.has_public_photos():
            changed += self.the_parent.make_private()
        return changed

    def make_public(self):
        if not self.private:
            return None
        if self.the_parent and self.the_parent.private:
            return None
        self.private = False
        self.save()
        photos = self.get_photos()
        if photos:
            for photo in photos:
                if photo.private:
                    photo.private = False
                    photo.save()
        else:
            for subalbum in self.get_subalbums():
                if subalbum.private:
                    subalbum.make_public()
        return self

    def do_delete(self):
        if not self.the_parent:
            return self._delete_year()
        if not self.the_parent.the_parent:
            return self._delete_month()
        if not self.the_parent.the_parent.the_parent:
            return self._delete_event()

    def __str__(self):
        s = '%s (%s): %s-%s' % (self.name, self.slug,
                                self.first_timestamp.isoformat(),
                                self.last_timestamp.isoformat())
        a = self.the_parent
        while a:
            s = '%s > %s' % (a.slug, s)
            a = a.the_parent
        return s

    def _set_cover_event(self):
        ma, ya = self.the_parent, self.the_parent.the_parent
        if ma.the_photo != self.the_photo:
            old = ma.the_photo
            if ya.the_photo == old:
                ya.the_photo = self.the_photo
                ya.save()
            ma.the_photo = self.the_photo
            ma.save()

    def _set_cover_month(self):
        ya = self.the_parent
        if ya.the_photo != self.the_photo:
            ya.the_photo = self.the_photo
            ya.save()

    def _get_prev_year(self):
        years = Album.get_years()
        if not years or years.first() == self:
            return None
        i = 1
        while years[i] != self:
            i += 1
        return years[i-1]

    def _get_next_year(self):
        years = Album.get_years()
        if not years or self == years.last():
            return None
        i = years.count() - 2 # Previous to last
        while years[i] != self:
            i -= 1
        return years[i+1]

    def _get_prev_month(self):
        months = self.the_parent.get_subalbums()
        if self == months.first():
            prev_year = self.the_parent._get_prev_year()
            if not prev_year:
                return None
            prev_year_months = prev_year.get_subalbums()
            return prev_year_months.last()
        i = 1
        while months[i] != self:
            i += 1
        return months[i-1]

    def _get_next_month(self):
        months = self.the_parent.get_subalbums()
        if self == months.last():
            next_year = self.the_parent._get_next_year()
            if not next_year:
                return None
            next_year_months = next_year.get_subalbums()
            return next_year_months.first()
        i = months.count() - 2 # Previous to last
        while months[i] != self:
            i -= 1
        return months[i+1]

    def _get_prev_event(self):
        events = self.the_parent.get_subalbums()
        if self == events.first():
            prev_month = self.the_parent._get_prev_month()
            if not prev_month:
                return None
            prev_month_events = prev_month.get_subalbums()
            return prev_month_events.last()
        i = 1
        while events[i] != self:
            i += 1
        return events[i-1]

    def _get_next_event(self):
        events = self.the_parent.get_subalbums()
        if self == events.last():
            next_month = self.the_parent._get_next_month()
            if not next_month:
                return None
            next_month_events = next_month.get_subalbums()
            return next_month_events.first()
        i = events.count() - 2 # Previous to last
        while events[i] != self:
            i -= 1
        return events[i+1]

    def _reset(self, old_cover, new_cover):
        self.refresh_from_db()
        if self.the_photo == old_cover:
            self.the_photo = new_cover
            self.save()

    def _reset_event(self):
        old_cover = self.the_photo
        new_cover = self.get_photos().first()
        self.the_parent.the_parent._reset(old_cover, new_cover)
        self.the_parent._reset(old_cover, new_cover)
        self._reset(old_cover, new_cover)

    def _reset_month(self):
        events = self.get_subalbums()
        for event in events:
            event.the_photo = event.get_photos().first()
            event.save()
        old_cover = self.the_photo
        new_cover = events.first().the_photo
        self.the_parent._reset(old_cover, new_cover)
        self._reset(old_cover, new_cover)

    def _reset_year(self):
        months = self.get_subalbums()
        for month in months:
            events = month.get_subalbums()
            for event in events:
                event.the_photo = event.get_photos().first()
                event.save()
            month.the_photo = events.first().the_photo
            month.save()
        old_cover = self.the_photo
        new_cover = months.first().the_photo
        self._reset(old_cover, new_cover)

    def _delete_year(self):
        months = self.get_subalbums()
        events = Album.objects.filter(the_parent__the_parent=self)
        photos = Photo.objects.filter(the_album__the_parent__the_parent=self)

        for album in Album.has_cover_in(photos):
            album.the_photo = None
            album.save()
        for photo in photos:
            photo.delete()
        for event in events:
            event.delete()
        for month in months:
            month.delete()

        ys = self.slug
        self.delete()
        Album._delete_year_tree(ys)

    def _delete_month(self):
        events = self.get_subalbums()
        photos = Photo.objects.filter(the_album__the_parent=self)

        for album in Album.has_cover_in(photos):
            album.the_photo = None
            album.save()
        for photo in photos:
            photo.delete()
        for event in events:
            event.delete()

        ms = self.slug
        year = self.the_parent
        self.delete()
        Album._delete_month_tree(year.slug, ms)

        year.refresh_from_db()
        months = year.get_subalbums()
        if months.count() == 0:
            ys = year.slug
            year.delete()
            Album._delete_year_tree(ys)
            year = None
        elif year.the_photo is None:
            year.the_photo = months.first().the_photo
            year.save()
        return year

    def _delete_event(self):
        locations = []

        photos = self.get_photos()

        for album in Album.has_cover_in(photos):
            album.the_photo = None
            album.save()
        for photo in photos:
            photo.delete()

        es = self.slug
        month = self.the_parent
        year = self.the_parent.the_parent
        self.delete()
        Album._delete_event_tree(year.slug, month.slug, es)

        month.refresh_from_db()
        events = month.get_subalbums()
        if events.count() == 0:
            ms = month.slug
            Album._delete_month_tree(year.slug, ms)
            month.delete()
            month = None
        elif month.the_photo is None:
            month.the_photo = events.first().the_photo
            mont.save()

        year.refresh_from_db()
        months = year.get_subalbums()
        if months.count() == 0:
            ys = year.slug
            year.delete()
            Album._delete_year_tree(ys)
            year = None
        elif year.the_photo is None:
            year.the_photo = months.first().the_photo
            year.save()

        return year, month

    @classmethod
    def get_years(klass, reverse=False):
        order = '-first_timestamp' if reverse else 'first_timestamp'
        return Album.objects.filter(the_parent=None).order_by(order)

    @classmethod
    def has_public_years(klass):
        for year in Album.objects.filter(the_parent=None)\
                                 .order_by('first_timestamp'):
           if not year.private:
               return True
        return False

    @classmethod
    def get_year(klass, year):
        return Album.objects.filter(the_parent=None, slug=year).first()

    @classmethod
    def get_month(klass, year, month):
        ya = Album.get_year(year)
        return ya, Album.objects.filter(the_parent=ya, slug=month).first()

    @classmethod
    def get_event(klass, year, month, event):
        ya, ma = Album.get_month(year, month)
        return ya, ma, Album.objects.filter(the_parent=ma, slug=event).first()

    @classmethod
    def get_random_photo(klass, blacklist=None):
        photo = Photo.objects.filter(private=False).order_by('?').first()
        if not photo or not blacklist:
            return photo
        terms = [ t.lower() for t in blacklist.split(',') ]
        blacklisted = True
        while blacklisted:
            photo = Photo.objects.filter(private=False).order_by('?').first()
            blacklisted = False
            for term in terms:
                if photo.matches(term):
                    blacklisted = True
                    continue
        return photo

    @classmethod
    def has_cover_in(klass, photos):
        albums = []
        for photo in photos:
            for album in Album.objects.filter(the_photo=photo):
                albums.append(album)
        return albums

    @classmethod
    def _add_term_to_query(query, q):
        if query:
            return query | q
        return q

    @classmethod
    def _get_bound_time(klass, bound):
        t = bound.split(':')
        tz = pytz.timezone(TIME_ZONE)
        dt = datetime.strptime(t[1], '%Y/%m/%d')
        return datetime(dt.year, dt.month, dt.day, dt.hour, \
                        dt.minute, dt.second, dt.microsecond, tz)

    @classmethod
    def _add_query_term(klass, query, s):
        q = (Q(title__icontains=s) | \
             Q(description__icontains=s) | \
             Q(basename__icontains=s) | \
             Q(the_album__name__icontains=s))
        if query:
            return query | q
        return q

    @classmethod
    def _add_query_neg_term(klass, query, s):
        q = (Q(title__icontains=s) | \
             Q(description__icontains=s) | \
             Q(basename__icontains=s) | \
             Q(the_album__name__icontains=s))
        if query:
            return query | q
        return q

    @classmethod
    def search(klass, s, realm=''):
        empty = Photo.objects.filter(id=-1)
        if s == '':
            return empty
        photos = Photo.objects.all().order_by('timestamp')
        phrases = []
        nphrases = []
        query = None
        nquery = None
        while '"' in s:
            p, sign = getquote(s)
            if p is None:
                break
            if sign:
                s = s.replace(p, '', 1)
            else:
                s = s.replace('-' + p, '', 1)
            p = p[1:-1].strip()
            if p == '':
                continue
            if sign:
                phrases.append(p)
                query = Album._add_query_term(query, p)
            else:
                nphrases.append(p)
                nquery = Album._add_query_neg_term(nquery, p)
        terms = s.split(' ')
        before = after = None
        sterms = phrases
        snterms = nphrases
        try:
            for t in terms:
                if t == '':
                    continue
                elif t.startswith('before:'):
                    before = Album._get_bound_time(t)
                elif t.startswith('after:'):
                    after = Album._get_bound_time(t)
                elif t.startswith('-'):
                    nquery = Album._add_query_neg_term(nquery, t[1:])
                    snterms.append(t)
                else:
                    query = Album._add_query_term(query, t)
                    sterms.append(t)
        except ValueError:
            return empty
        t = realm.split('/')
        if len(t) > 3:
            return empty
        if not before and not after and not query:
            return empty
        if before:
            photos = photos.filter(timestamp__lt=before)
        if after:
            photos = photos.filter(timestamp__gt=after)
        if not query and not nquery:
            return photos

        if query:
            photos = photos.filter(query)
        if nquery:
            photos = photos.exclude(nquery)
        if realm == '':
            return photos
        if len(t) == 1:
            ya = Album.get_year(t[0])
            if not ya:
                return empty
            return photos.filter(the_album__the_parent__the_parent=ya)
        if len(t) == 2:
            ya, ma = Album.get_month(t[0], t[1])
            if not ya or not ma:
                return empty
            return photos.filter(the_album__the_parent__the_parent=ya, \
                                 the_album__the_parent=ma)
        if len(t) == 3:
            ya, ma, ea = Album.get_event(t[0], t[1], t[2])
            if not ya or not ma or not ea:
                return empty
            return photos.filter(the_album__the_parent__the_parent=ya, \
                                 the_album__the_parent=ma,
                                 the_album=ea)
        # Unreachable
        return empty

    @classmethod
    def reset_all(klass):
        for year in Album.get_years():
            year.reset()

    @classmethod
    def full_tree(klass, auth=False):
        tree = {}
        years = Album.objects.filter(the_parent=None).order_by('first_timestamp')
        for year in years:
            if not auth and year.private:
                continue
            tree[year.slug] = { 'album': year,
                                'label': year.get_day_range(),
                                'tree': year.get_tree() }
        return tree

    @classmethod
    def set_covers(klass, covers):
        for year in covers.keys():
            ya = Album.get_year(year)
            if ya is None:
                raise ValueError('year', year)
            months = covers[year]['months']
            for month in months.keys():
                ma = ya.get_subalbum(month)
                if ma is None:
                    raise ValueError('month', month)
                events = months[month]['events']
                for event in events.keys():
                    ea = ma.get_subalbum(event)
                    if ea is None:
                        raise ValueError('month', month)
                    photo = events[event]['cover']
                    ph = ea.get_photo(photo)
                    if ph is None:
                        raise ValueError('event', event)
                    ea.the_photo = ph
                    ea.save()
                for event in events.keys():
                    ea = ma.get_subalbum(event)
                    if events[event]['is_cover']:
                        ma.the_photo = ea.the_photo
                        ma.save()
            for month in months.keys():
                ma = ya.get_subalbum(month)
                if months[month]['is_cover']:
                    ya.the_photo = ma.the_photo
                    ya.save()

    @classmethod
    def get_covers(klass):
        r = {}
        years = Album.get_years()
        for year in years:
            if year.private:
                continue
            r[year.slug] = { 'months': {} }
            months = year.get_subalbums()
            for month in months:
                if month.private:
                    continue
                ms = r[year.slug]['months']
                ms[month.slug] = { 'events': {} }
                events = month.get_subalbums()
                for event in events:
                    if event.private or event.the_photo.private:
                        continue
                    es = ms[month.slug]['events']
                    es[event.slug] = { 'cover': event.the_photo.basename }
                    es[event.slug]['is_cover'] = \
                        month.the_photo == event.the_photo
                ms[month.slug]['is_cover'] = \
                    year.the_photo == month.the_photo
        return r

    @classmethod
    def _delete_year_tree(klass, year):
        prefix = os.path.join(STATIC_PATH_PREFIX, 'galeria')
        for size in [ 'photos', 'exports', 'views', 'thumbs' ]:
            location = os.path.join(prefix, size, year)
            if os.path.exists(location):
                rmtree(location)

    @classmethod
    def _delete_month_tree(klass, year, month):
        prefix = os.path.join(STATIC_PATH_PREFIX, 'galeria')
        for size in [ 'photos', 'exports', 'views', 'thumbs' ]:
            location = os.path.join(prefix, size, year, month)
            if os.path.exists(location):
                rmtree(location)

    @classmethod
    def _delete_event_tree(klass, year, month, event):
        prefix = os.path.join(STATIC_PATH_PREFIX, 'galeria')
        for size in [ 'photos', 'exports', 'views', 'thumbs' ]:
            location = os.path.join(prefix, size, year, month, event)
            if os.path.exists(location):
                rmtree(location)

    class Meta:
        indexes = [ models.Index(fields=['the_parent',
                                         'slug',
                                         'first_timestamp',
                                         'last_timestamp',
                                         'the_photo']) ]
        ordering = ('first_timestamp',)

class Photo(models.Model):

    title = models.CharField(max_length=60)
    description = models.CharField(max_length=250)
    the_album = models.ForeignKey('Album', on_delete=models.CASCADE)
    location = models.CharField(max_length=250)
    latitude = models.FloatField(default=sys.float_info.max)
    longitude = models.FloatField(default=sys.float_info.max)
    width = models.SmallIntegerField(default=-1)
    height = models.SmallIntegerField(default=-1)
    export_width = models.SmallIntegerField(default=-1)
    export_height = models.SmallIntegerField(default=-1)
    view_width = models.SmallIntegerField(default=-1)
    view_height = models.SmallIntegerField(default=-1)
    thumb_width = models.SmallIntegerField(default=-1)
    thumb_height = models.SmallIntegerField(default=-1)
    timestamp = models.DateTimeField()
    view_timestamp = models.CharField(max_length=60)
    time_offset = models.SmallIntegerField(default=-1)
    private = models.BooleanField(default=False)

    url = models.CharField(max_length=250, null=True)
    export_url = models.CharField(max_length=250, null=True)
    view_url = models.CharField(max_length=250, null=True)
    thumb_url = models.CharField(max_length=250, null=True)
    basename = models.CharField(max_length=100, null=True)
    view_latitude = models.CharField(max_length=50, null=True)
    view_longitude = models.CharField(max_length=50, null=True)

    def fill_aux_data(self):
        self.url = self.location.replace(STATIC_PATH_PREFIX, '')
        self.export_url = self.url.replace('/photos/', '/exports/', 1)
        self.view_url = self.url.replace('/photos/', '/views/', 1)
        self.thumb_url = self.url.replace('/photos/', '/thumbs/', 1)
        self.basename = os.path.basename(self.location)
        locale =  LANGUAGE_CODE.replace('-' , '_')
        self.view_timestamp = format_datetime(
            self.timestamp, locale=locale,
            format='MMMM d, yyyy, HH:mm:ss [zzz]').capitalize()
        vlat, vlon = get_view_coords(self.latitude, self.longitude)
        self.view_latitude, self.view_longitude = vlat, vlon

    def localdatetime(self):
        tz = timezone(timedelta(hours=self.time_offset))
        return datetime(self.timestamp.year, self.timestamp.month,
                        self.timestamp.day, self.timestamp.hour,
                        self.timestamp.minute, self.timestamp.second,
                        self.timestamp.microsecond,
                        timezone(timedelta(hours=self.time_offset)))

    def get_url_path(self):
        return os.path.join(self.the_album.get_url_path(), self.basename)

    def matches(self, term):
        return ((self.title and term in self.title.lower()) or
                (self.description and term in self.description.lower()) or
                (self.the_album.name and term in self.the_album.name.lower()))

    def get_prev_url(self):
        prev = self.get_prev()
        if prev:
            return prev.get_url_path()
        return None

    def get_next_url(self):
        next = self.get_next()
        if next:
            return next.get_url_path()
        return None

    def get_prev(self):
        photos = self.the_album.get_photos()
        if self == photos.first():
            prev_event = self.the_album.get_prev()
            if not prev_event:
                return None
            prev_event_photos = prev_event.get_photos()
            return prev_event_photos.last()
        i = 1
        while i < photos.count() and photos[i] != self:
            i += 1
        return photos[i-1]

    def get_next(self):
        photos = self.the_album.get_photos()
        if self == photos.last():
            next_event = self.the_album.get_next()
            if not next_event:
                return None
            next_event_photos = next_event.get_photos()
            return next_event_photos.first()
        i = photos.count() - 2 # Previous to last
        while i > 0 and photos[i] != self:
            i -= 1
        return photos[i+1]

    def set_cover(self):
        if self.the_album.the_photo != self:
            ya, ma, ea = self.the_album.the_parent.the_parent, \
                self.the_album.the_parent, self.the_album
            old = ea.the_photo
            if ya.the_photo == old:
                ya.the_photo = self
                ya.save()
            if ma.the_photo == old:
                ma.the_photo = self
                ma.save()
            ea.the_photo = self
            ea.save()

    def make_private(self):
        if self.private:
            return []
        self.private = True
        self.save()
        changed = [ self ]
        if not self.the_album.has_public_photos():
            changed += self.the_album.make_private()
        return changed

    def make_public(self):
        if not self.private:
            return None
        if self.the_album.private:
            return None
        self.private = False
        self.save()
        return self

    def contained_in(self, album):
        return self.the_album == album or \
            self.the_album.the_parent == album or\
            self.the_album.the_parent.the_parent == album

    def __str__(self):
        s = '%s (%s): %04d/%02d/%02d %02d:%02d:%02d' % \
            (self.title, self.basename,
             self.timestamp.year, self.timestamp.month,
             self.timestamp.day, self.timestamp.hour,
             self.timestamp.minute, self.timestamp.second)

        a = self.the_album
        while a:
            s = '%s > %s' % (a.slug, s)
            a = a.the_parent
        return s

    def do_delete(self, token):
        ya, ma, ea = self.the_album.the_parent.the_parent, \
            self.the_album.the_parent, self.the_album

        for album in Album.has_cover_in([self]):
            album.the_photo = None
            album.save()
        free(self, token)

        loc = self.location
        ys, ms, es = ea.the_parent.the_parent.slug, ea.the_parent.slug, ea.slug
        self.delete()

        ea.refresh_from_db()
        photos = ea.get_photos()
        if photos.count() == 0:
            ea.delete()
            ea = None
            Album._delete_event_tree(ys, ms, es)
        elif ea.the_photo is None:
            Photo._delete_locations(loc)
            ea.the_photo = photos.first()
            ea.save()

        ma.refresh_from_db()
        events = ma.get_subalbums()
        if events.count() == 0:
            ma.delete()
            ma = None
            Album._delete_month_tree(ys, ms)
        elif ma.the_photo is None:
            ma.the_photo = events.first().the_photo
            ma.save()

        ya.refresh_from_db()
        months = ya.get_subalbums()
        if months.count() == 0:
            ya.delete()
            ya = None
            Album._delete_year_tree(ys)
        elif ya.the_photo is None:
            ya.the_photo = months.first().the_photo
            ya.save()

        return ya, ma, ea

    @classmethod
    def _delete_locations(klass, location):
        locations = [ location ]
        for size in [ 'exports', 'views', 'thumbs' ]:
            repl = '/%s/' % size
            locations.append(location.replace('/photos/', repl, 1))

        for location in locations:
            os.remove(location)

    @classmethod
    def get_photo(klass, year, month, event, photo):
        ya, ma, ev = Album.get_event(year, month, event)
        ph = Photo.objects \
                  .filter(the_album=ev, basename=photo) \
                  .order_by('timestamp').first()
        return ya, ma, ev, ph

    @classmethod
    def _get_photo_album(klass, metadata):
        ts = metadata['timestamp']
        offset = metadata['time_offset']
        sy = '%04d' % ts.year
        sm = '%02d' % ts.month
        year = Album.objects.filter(the_parent=None, slug=sy).first()
        if not year:
            year = Album(the_parent = None,
                         name = get_year_name(sy),
                         slug = sy,
                         first_timestamp = ts,
                         last_timestamp = ts,
                         time_offset = metadata['time_offset'],
                         the_photo = None)
            year.save()
        month = Album.objects.filter(the_parent=year, slug=sm).first()
        if not month:
            month = Album(the_parent = year,
                          name = get_month_name(ts, offset, True),
                          slug = sm,
                          first_timestamp = ts,
                          last_timestamp = ts,
                          time_offset = metadata['time_offset'],
                          the_photo = None)
            month.save()
        slug = normalize(metadata['album'])
        event = Album.objects.filter(the_parent=month, slug=slug).first()
        if not event:
            event = Album(the_parent = month,
                                name = metadata['album'],
                                slug = slug,
                                first_timestamp = ts,
                                last_timestamp = ts,
                                time_offset = metadata['time_offset'],
                                the_photo = None)
            event.save()
        return year, month, event

    @classmethod
    def add_photo(klass, uploaded_image, token=None):
        path = str(uploaded_image.files.file)
        uploaded_image.delete()
        md = get_image_metadata(path)
        if not md:
            os.remove(path)
            return
        year, month, event = Photo._get_photo_album(md)
        photo = Photo(title = md['title'],
                      description = md['description'],
                      the_album = event,
                      location = md['location'],
                      latitude = md['latitude'],
                      longitude = md['longitude'],
                      width = md['width'],
                      height = md['height'],
                      export_width = md['export_width'],
                      export_height = md['export_height'],
                      view_width = md['view_width'],
                      view_height = md['view_height'],
                      thumb_width = md['thumb_width'],
                      thumb_height = md['thumb_height'],
                      timestamp = md['timestamp'],
                      time_offset = md['time_offset'])
        photo.fill_aux_data()
        photo.save()
        if event.first_timestamp > photo.timestamp:
            event.first_timestamp = photo.timestamp
        if event.last_timestamp < photo.timestamp:
            event.last_timestamp = photo.timestamp
        if not event.the_photo:
            event.the_photo = photo
        event.save()
        if month.first_timestamp > event.first_timestamp:
            month.first_timestamp = event.first_timestamp
        if month.last_timestamp < event.last_timestamp:
            month.last_timestamp = event.last_timestamp
        if not month.the_photo:
            month.the_photo = photo
        month.save()
        if year.first_timestamp > month.first_timestamp:
            year.first_timestamp = month.first_timestamp
        if year.last_timestamp < month.last_timestamp:
            year.last_timestamp = month.last_timestamp
        if not year.the_photo:
            year.the_photo = photo
        year.save()
        os.remove(path)
        if token:
            restrict(photo.make_private(), token)

    class Meta:
        indexes = [ models.Index(fields=['timestamp',
                                         'location']) ]
        ordering = ('timestamp',)

class PhotoModerator(CommentModerator):

    def email(self, comment, content_object, request):
        photo = content_object
        album = photo.the_album
        subject = 'New comment for photo "%s" in album "%s"' % \
            (photo.title, album.name)
        body = (('A new comment for the photo "%s" ' % photo.title) + \
                ('in the album "%s" has been posted.\n\n' % album.name) + \
                ('Author: %s\n' % comment.user_name) + \
                ('Email: %s\n' % comment.user_email) + \
                ('Comment:\n\n' + comment.comment + '\n\n') +  \
                ('Please go to the administrative interface to ' + \
                 'moderate it.\n'))
        send_mail(subject, body, SYSTEM_EMAIL, [GALERIA_INFO['email']],
                  fail_silently=False)

    def moderate(self, comment, content_object, request):
        return True

class UploadedImage(models.Model):

    files = models.ImageField(upload_to=UPLOAD_TMP_DIR,
                              blank=True, null=True)

class UploadedFile(models.Model):

    file = models.FileField(upload_to=UPLOAD_TMP_DIR,
                            blank=True, null=True)

moderator.register(Photo, PhotoModerator)
