import os
import os.path
import sys
import json

from django.contrib.auth import get_user, authenticate, \
    logout as django_logout, login as django_login
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator, InvalidPage
from django.http import HttpResponse, Http404, \
    HttpResponseBadRequest,  JsonResponse
from django.shortcuts import render, redirect
from django.utils.http import urlencode
from django.utils.translation import ngettext, gettext as _, ngettext
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.edit import FormView

from rest_framework.authentication import TokenAuthentication
from rest_framework.authtoken.models import Token
from rest_framework.decorators import api_view, authentication_classes, \
    parser_classes, permission_classes
from rest_framework.parsers import JSONParser
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response

from django_comments.models import Comment

from galeria.settings import GALERIA_INFO, PAGINATION_ITEMS, LIST_ITEMS, \
    ALBUMS_ORDER, PHOTOS_ORDER, SERVER_URL, FORCE_SCRIPT_NAME

from fotos.models import Album, Photo, UploadedFile, UploadedImage
from fotos.forms import UploadedImageForm, UploadedFileForm
from fotos.util import get_month_name, restrict, free
from fotos.serializers import AlbumSerializer, PhotoSerializer, \
    UploadedImageSerializer

from datetime import timezone, timedelta

def _get_ask_user(title, message, ok, cancel, link):
    return "askUser('%s', '%s', '%s', '%s', '%s')" % \
        (title, message, ok, cancel, link);

def _get_delete_redirect(ya = None, ma = None, ea = None):
        if ea:
            return ea.get_url_path()
        if ma:
            return ma.get_url_path()
        if ya:
            return ya.get_url_path()
        return FORCE_SCRIPT_NAME + '/'

def _log_error(message):
    sys.stderr.write(message)
    return None

def _delete(path, token):
    t = path.split('/')
    ya = ma = ea = ph = None
    if len(t) == 4:
        ya, ma, ea, ph = Photo.get_photo(*t)
    if len(t) == 3:
        ya, ma, ea = Album.get_event(*t)
    if len(t) == 2:
        ya, ma = Album.get_month(*t)
    if len(t) == 1:
        ya = Album.get_year(t[0])

    err_message = _('Error deleting: %(error)s\n')
    if ya and ma and ea and ph:
        try:
            ya, ma, ea = ph.do_delete(token)
        except Exception as e:
            return _log_error(err_message % { 'error': str(e) })
        return _get_delete_redirect(ya, ma, ea)
    if ya and ma and ea:
        try:
            ya, ma = ea.do_delete()
        except Exception as e:
            return _log_error(err_message % { 'error': str(e) })
        return _get_delete_redirect(ya, ma)
    if ya and ma:
        try:
            ya = ma.do_delete()
        except Exception as e:
            return _log_error(err_message % { 'error': str(e) })
        return _get_delete_redirect(ya)
    if ya:
        try:
            ya.do_delete()
        except Exception as e:
            return _log_error(err_message % { 'error': str(e) })
        return FORCE_SCRIPT_NAME + '/'
    return None

def get_moderated_comments():
    return Comment.objects.filter(is_public=False)

def get_approved_comments():
    return Comment.objects.filter(is_public=True)

def index(request):
    link = message = ''
    page_title = 'Galería'
    albums = Album.get_years(ALBUMS_ORDER == 'descendant')
    if not Album.has_public_years() or albums.count() == 0:
        page_title = _('The gallery is empty')
        if request.user.is_authenticated:
            link = FORCE_SCRIPT_NAME + '/control/upload'
            message = _('Start uploading photographs')
        else:
            link = FORCE_SCRIPT_NAME + '/control/login'
            message = _('Login into the system to upload photographs')
        summary = _('There is no uploaded photographs')
    else:
        n_years = albums.count()
        summary = (ngettext('One year', '%(years)s years', n_years) % \
                   { 'years': f'{n_years:n}' })

    d_title = _('Reset all covers')
    d_message = _('This will reset all the gallery covers. Are you sure?')
    d_ok = _('Reset covers')
    d_cancel = _('Cancel')
    reset_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                  FORCE_SCRIPT_NAME + '/control/reset?' +
                                  urlencode({'path': '/'}))

    d_title = _('Delete gallery')
    d_message = _('This will delete all the photographs ' \
                  'from the gallery forever. Are you sure?')
    d_ok = _('Delete photographs')
    d_cancel = _('Cancel')
    delete_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                   FORCE_SCRIPT_NAME + '/control/delete?' +
                                   urlencode({'path': '/'}))

    attrs = { 'albums': albums,
              'albumtree': Album.full_tree(request.user.is_authenticated),
              'breadcrumbs': [],
              'galeria': GALERIA_INFO,
              'link': link,
              'menus': [{ 'confirm': True,
                          'icon': 'window-restore',
                          'id': 'menu-reset-cover',
                          'name': _('Reset covers in the gallery'),
                          'onclick': reset_onclick },
                        { 'confirm': True,
                          'icon': 'trash',
                          'id': 'menu-delete',
                          'name': _('Delete gallery'),
                          'onclick': delete_onclick }],
              'message': message,
              'navigation': { 'prev': None, 'next': None, 'up': None },
              'page_title': page_title,
              'path': FORCE_SCRIPT_NAME,
              'prefix': FORCE_SCRIPT_NAME,
              'search': { 'realm': '' },
              'summary': summary,
              'moderated': get_moderated_comments(),
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def year(request, year):
    ya = Album.get_year(year)
    if not ya:# or (ya.private and not request.user.is_authenticated):
        raise Http404(year + ' does not exist')

    months = ya.get_subalbums(ALBUMS_ORDER == 'descendant')
    n_months = months.count()
    summary = (ngettext('One month', '%(months)s months', n_months) %
               { 'months': f'{n_months:n}' })

    path = ya.get_url_path().replace(FORCE_SCRIPT_NAME + '/', '', 1)

    d_title = _('Reset year covers')
    d_message = _('This will reset all the year covers. Are you sure?')
    d_ok = _('Reset covers')
    d_cancel = _('Cancel')
    reset_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                  FORCE_SCRIPT_NAME + '/control/reset?' +
                                  urlencode({'path': path}))

    d_title = _('Delete year')
    d_message = _('This will delete all the photographs ' \
                  'from the year forever. Are you sure?')
    d_ok = _('Delete photographs')
    d_cancel = _('Cancel')
    delete_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                   FORCE_SCRIPT_NAME + '/control/delete?' +
                                   urlencode({'path': path}))

    if ya.private:
        access_icon = 'unlock'
        access_name = _('Make year public')
        access_url = FORCE_SCRIPT_NAME + '/control/public?' + \
            urlencode({'path': path})
    else:
        access_icon = 'lock'
        access_name = _('Make year private')
        access_url = FORCE_SCRIPT_NAME + '/control/private?' + \
            urlencode({'path': path})

    attrs = { 'album': ya,
              'albums': months,
              'albumtree': ya.get_tree(request.user.is_authenticated),
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name } ],
              'galeria': GALERIA_INFO,
              'cover': ya.the_photo.the_album.the_parent,
              'locked': ya.private,
              'menus': [{ 'confirm': True,
                          'icon': 'window-restore',
                          'id': 'menu-reset-cover',
                          'name': _('Reset year covers'),
                          'onclick': reset_onclick},
                        { 'link': access_url,
                          'icon': access_icon,
                          'id': 'menu-access',
                          'name': access_name },
                        { 'confirm': True,
                          'icon': 'trash',
                          'id': 'menu-delete',
                          'name': _('Delete year'),
                          'onclick': delete_onclick }],
              'navigation': { 'prev': ya.get_prev_url(),
                              'next': ya.get_next_url(),
                              'up': FORCE_SCRIPT_NAME + '/#a-' + year },
              'page_title': _('Year %(year)s') % { 'year': year },
              'path': FORCE_SCRIPT_NAME + '/' + year,
              'prefix': FORCE_SCRIPT_NAME,
              'search': { 'realm': year },
              'summary': summary,
              'moderated': get_moderated_comments(),
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def month(request, year, month):
    ya, ma = Album.get_month(year, month)
    if not ya or not ma:
        raise Http404(os.path.join(year, month) + ' does not exist')

    events = ma.get_subalbums(ALBUMS_ORDER == 'descendant')
    n_events = events.count()
    summary = (ngettext('One event', '%(events)s events', n_events) %
               { 'events': f'{n_events:n}' })

    page_title = (_('Month of %(month)s, %(year)s') % \
                   { 'month': get_month_name(ma.first_timestamp, \
                                             ma.time_offset),
                     'year': ya.slug })

    path = ma.get_url_path().replace(FORCE_SCRIPT_NAME + '/', '', 1)

    d_title = _('Reset month covers')
    d_message = _('This will reset all the month covers. Are you sure?')
    d_ok = _('Reset covers')
    d_cancel = _('Cancel')
    reset_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                  FORCE_SCRIPT_NAME + '/control/reset?' +
                                  urlencode({'path': path}))

    d_title = _('Delete month')
    d_message = _('This will delete all the photographs '
                  'from the month forever. Are you sure?')
    d_ok = _('Delete photographs')
    d_cancel = _('Cancel')
    delete_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                   FORCE_SCRIPT_NAME + '/control/delete?' +
                                   urlencode({'path': path}))

    set_cover_url = FORCE_SCRIPT_NAME + '/control/set-cover?' + \
        urlencode({'path': path})
    if ma.private:
        access_icon = 'unlock'
        access_name = _('Make month public')
        access_url = FORCE_SCRIPT_NAME + '/control/public?' + \
            urlencode({'path': path})
    else:
        access_icon = 'lock'
        access_name = _('Make month private')
        access_url = FORCE_SCRIPT_NAME + '/control/private?' + \
            urlencode({'path': path})

    page = request.GET.get('page', 1)
    paginator = Paginator(events, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)

    attrs = { 'album': ma,
              'albums': the_page,
              'albumtree': ma.get_tree(request.user.is_authenticated),
              'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name } ],
              'galeria': GALERIA_INFO,
              'cover': ma.the_photo.the_album,
              'is_cover': ya.the_photo == ma.the_photo,
              'locked': ma.private,
              'menus': [{ 'confirm': True,
                          'icon': 'window-restore',
                          'id': 'menu-reset-cover',
                          'name': _('Reset month covers'),
                          'onclick': reset_onclick },
                        { 'link': set_cover_url,
                          'icon': 'check-circle',
                          'id': 'menu-set-cover',
                          'name': _('Set month as cover') },
                        { 'link': access_url,
                          'icon': access_icon,
                          'id': 'menu-access',
                          'name': access_name },
                        { 'confirm': True,
                          'icon': 'trash',
                          'id': 'menu-delete',
                          'name': _('Delete month'),
                          'onclick': delete_onclick }],
              'navigation': { 'prev': ma.get_prev_url(),
                              'next': ma.get_next_url(),
                              'up':  ya.get_url_path() + '#a-' + month },
              'page': the_page,
              'page_title': page_title,
              'path': os.path.join(FORCE_SCRIPT_NAME + '/', year, month),
              'prefix': FORCE_SCRIPT_NAME,
              'search': { 'realm': os.path.join(year, month) },
              'summary': summary,
              'moderated': get_moderated_comments(),
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def event(request, year, month, event):
    ya, ma, ea = Album.get_event(year, month, event)
    if not ya or not ma or not ea or \
       (ea.private and not request.user.is_authenticated):
        raise Http404(os.path.join(year, month, event) + ' does not exist')

    photos = ea.get_photos()
    n_photos = photos.count()
    summary = (ngettext('One photograph', '%(photos)s photographs', n_photos) %
               { 'photos': f'{n_photos:n}' })

    path = ea.get_url_path().replace(FORCE_SCRIPT_NAME + '/', '', 1)
    map_url = FORCE_SCRIPT_NAME + '/control/map?' + urlencode({'path': path})

    d_title = _('Reset event cover')
    d_message = _('This will reset the event cover. Are you sure?')
    d_ok = _('Reset cover')
    d_cancel = _('Cancel')
    reset_onclick = _get_ask_user(d_title, d_message,d_ok, d_cancel,
                                  FORCE_SCRIPT_NAME + '/control/reset?' +
                                  urlencode({'path': path}))

    d_title = _('Delete event')
    d_message = _('This will delete all the photographs '
                  'from the event forever. Are you sure?')
    d_ok = _('Delete photographs')
    d_cancel = _('Cancel')
    delete_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                   FORCE_SCRIPT_NAME + '/control/delete?' +
                                   urlencode({'path': path}))

    set_cover_url = FORCE_SCRIPT_NAME + '/control/set-cover?' + \
        urlencode({'path': path})
    if ea.private:
        access_icon = 'unlock'
        access_name = _('Make event public')
        access_url = FORCE_SCRIPT_NAME + '/control/public?' + \
            urlencode({'path': path})
    else:
        access_icon = 'lock'
        access_name = _('Make event private')
        access_url = FORCE_SCRIPT_NAME + '/control/private?' + \
            urlencode({'path': path})

    up = ma.get_url_path()
    page = request.GET.get('page', 1)
    paginator = Paginator(photos, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)

    events = ma.get_subalbums(ALBUMS_ORDER == 'descendant')
    i = 0
    while ea != events[i]:
        i += 1
    up_page = 1 + int(float(i) / PAGINATION_ITEMS)
    if up_page > 1:
        up += '?page=%d' % up_page

    attrs = { 'album': ea,
              'breadcrumbs': [{ 'link': ya.get_url_path(),
                                'name': ya.name },
                              { 'link': ma.get_url_path(),
                                'name': ma.name },
                              { 'link': ea.get_url_path(),
                                'name': ea.name }],
              'galeria': GALERIA_INFO,
              'cover': ea.the_photo,
              'is_cover': ma.the_photo == ea.the_photo,
              'locked': ea.private,
              'menus': [{ 'link': map_url,
                          'icon': 'map',
                          'id': 'menu-map',
                          'name': _('Event map') },
                        { 'confirm': True,
                          'icon': 'window-restore',
                          'id': 'menu-reset-cover',
                          'name': _('Reset event cover'),
                          'onclick': reset_onclick },
                        { 'link': set_cover_url,
                          'icon': 'check-circle',
                          'id': 'menu-set-cover',
                          'name': _('Set event as cover') },
                        { 'link': access_url,
                          'icon': access_icon,
                          'id': 'menu-access',
                          'name': access_name },
                        { 'confirm': True,
                          'icon': 'trash',
                          'id': 'menu-delete',
                          'name': _('Delete event'),
                          'onclick': delete_onclick }],
              'navigation': { 'prev': ea.get_prev_url(),
                              'next': ea.get_next_url(),
                              'up': up + '#a-' + ea.slug },
              'page': the_page,
              'page_title': ea.name,
              'photos': the_page,
              'path': os.path.join(FORCE_SCRIPT_NAME + '/', year, month, event),
              'prefix': FORCE_SCRIPT_NAME,
              'results': None,
              'search': { 'realm': os.path.join(year, month, event) },
              'summary': summary,
              'moderated': get_moderated_comments(),
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def photo(request, year, month, event, photo):
    ya, ma, ea, ph = Photo.get_photo(year, month, event, photo)
    if not ya or not ma or not ea or not ph or \
       (ph.private and not request.user.is_authenticated):
        raise Http404(os.path.join(year, month, event, photo) +
                      ' does not exist')
    path = ph.get_url_path().replace(FORCE_SCRIPT_NAME + '/', '', 1)

    d_title = _('Delete photograph')
    d_message = _('This will delete the photograph forever. Are you sure?')
    d_ok = _('Delete photograph')
    d_cancel = _('Cancel')
    delete_onclick = _get_ask_user(d_title, d_message, d_ok, d_cancel,
                                   FORCE_SCRIPT_NAME + '/control/delete?' +
                                   urlencode({'path': path}))

    up = ea.get_url_path()
    photos = ea.get_photos(PHOTOS_ORDER == 'descendant')
    i = 0
    while ph != photos[i]:
        i += 1
    up_page = 1 + int(float(i) / PAGINATION_ITEMS)
    if up_page > 1:
        up += '?page=%d' % up_page

    set_cover_url = FORCE_SCRIPT_NAME + '/control/set-cover?' + \
        urlencode({'path': path})
    if ph.private:
        access_icon = 'unlock'
        access_name = _('Make photograph public')
        access_url = FORCE_SCRIPT_NAME + '/control/public?' + \
            urlencode({'path': path})
    else:
        access_icon = 'lock'
        access_name = _('Make photograph private')
        access_url = FORCE_SCRIPT_NAME + '/control/private?' + \
            urlencode({'path': path})

    attrs = { 'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name },
                               { 'link': ph.get_url_path(),
                                 'name': ph.title } ],
              'galeria': GALERIA_INFO,
              'is_cover': ea.the_photo == ph,
              'locked': ph.private,
              'menus': [{ 'link': set_cover_url,
                          'icon': 'check-circle',
                          'id': 'menu-set-cover',
                          'name': _('Set photograph as cover') },
                        { 'link': access_url,
                          'icon': access_icon,
                          'id': 'menu-access',
                          'name': access_name },
                        { 'confirm': True,
                          'icon': 'trash',
                          'id': 'menu-delete',
                          'name': _('Delete photograph'),
                          'onclick': delete_onclick }],
              'navigation': { 'prev': ph.get_prev_url(),
                              'next': ph.get_next_url(),
                              'up':  up + '#p-' + ph.basename },
              'page_title': ph.title,
              'photo': ph,
              'path': os.path.join(FORCE_SCRIPT_NAME, year, month, event),
              'prefix': FORCE_SCRIPT_NAME,
              'results': None,
              'moderated': get_moderated_comments(),
              'user': get_user(request) }
    return render(request, 'photo.html', attrs)

def search(request):
    s = request.GET.get('s', '')
    realm = request.GET.get('realm', '')
    page = request.GET.get('page', 1)
    link = message = ''
    photos = Album.search(s, realm)

    n = photos.count()
    if n == 0:
        link = FORCE_SCRIPT_NAME + '/'
        message = _('There are no photographs matching the search')
        page_title = _('Search with no results')
        summary = _('Check the search term')
    else:
        page_title = (_('Search results: “%(term)s”') %
                      { 'term': s })
        summary = (ngettext('One photograph', '%(photos)s photographs', n) % \
                   { 'photos': f'{n:n}' })

    qs = request.META.get('QUERY_STRING', '')

    paginator = Paginator(photos, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)

    bc_link = FORCE_SCRIPT_NAME + '/search/realm=%s&s=%s&page=%s' % \
        (realm, s, page)
    search_name = _('Search: “%(term)s”') % { 'term': s }
    attrs = { 'breadcrumbs': [{ 'link': request.get_full_path(),
                                'name': search_name }],
              'galeria': GALERIA_INFO,
              'link': link,
              'message': message,
              'navigation': { 'prev': None, 'next': None,
                              'up': FORCE_SCRIPT_NAME + '/' + realm },
              'page': the_page,
              'page_title': page_title,
              'path': FORCE_SCRIPT_NAME + '/result/?%s&path=' % qs,
              'results': the_page,
              'prefix': FORCE_SCRIPT_NAME,
              'summary': summary,
              'moderated': get_moderated_comments(),
              'user': get_user(request) }
    return render(request, 'album.html', attrs)

def result(request):
    path = request.GET.get('path', None)
    if not path:
        raise Http404('Missing path')
    t = path.split('/')
    if len(t) < 4:
        raise Http404('Ivalid path ' + path)

    realm = request.GET.get('realm', '')
    s = request.GET.get('s', '')
    page = request.GET.get('page', '1')
    photos = Album.search(s, realm)
    n = photos.count()
    if n == 0:
        raise Http404('Ivalid result ' + request.get_full_path())
    paginator = Paginator(photos, PAGINATION_ITEMS)
    the_page = paginator.get_page(page)
    np = len(the_page)

    year, month, event, photo = t[-4:]
    ya, ma, ea, ph = Photo.get_photo(year, month, event, photo)
    if not ya or not ma or not ea or not ph:
        raise Http404(os.path.join(year, month, event, photo +
                                   ' does not exist'))

    i = 0
    while ph != the_page[i]:
        i += 1

    prev = next = None
    p = FORCE_SCRIPT_NAME
    f = '%s/result/?realm=%s&s=%s&page=%s&path=%s%s'
    if np == 1:
        prev = next = None
    elif i == 0:
        if the_page.has_previous():
            pp = str(int(page)-1)
            prev_page = paginator.get_page(the_page.previous_page_number())
            prev = f % (p, realm, s, pp, p, prev_page[-1].get_url_path())
        next = f % (p, realm, s, page, p, the_page[i+1].get_url_path())
    elif i == len(the_page) - 1:
        prev = f % (p, realm, s, page, p, the_page[i-1].get_url_path())
        if the_page.has_next():
            np = str(int(page)+1)
            next_page = paginator.get_page(the_page.next_page_number())
            next = f % (p, realm, s, np, p, next_page[0].get_url_path())
    else:
        prev = f % (p, realm, s, page, p, the_page[i-1].get_url_path())
        next = f % (p, realm, s, page, p, the_page[i+1].get_url_path())

    up = FORCE_SCRIPT_NAME + '/search/?realm=%s&s=%s' % (realm, s)
    if page != '':
        up += '&page=%s' % page

    p = ph.get_url_path().replace(FORCE_SCRIPT_NAME + '/', '', 1)

    d_title = _('Delete photograph')
    d_message = _('This will delete the photograph forever. Are you sure?')
    d_ok = _('Delete photograph')
    d_cancel = _('Cancel')
    delete_onclick = _get_ask_user(d_title, d_title, d_ok, d_cancel,
                                   FORCE_SCRIPT_NAME + '/control/delete?' +
                                   urlencode({'path': p}))

    search_link = FORCE_SCRIPT_NAME + '/search/realm=%s&s=%s&page=%s' % \
        (realm, s, page)
    search_name = _('Search: “%(term)s”') % { 'term': s }

    attrs = { 'breadcrumbs': [ { 'link': search_link,
                                 'name': search_name },
                               { 'link': request.get_full_path(),
                                 'name': ph.title }],
              'galeria': GALERIA_INFO,
              'locked': ph.private,
              'menus': [{ 'confirm': True,
                          'icon': 'trash',
                          'id': 'menu-delete',
                          'name': _('Delete photograph'),
                          'onclick': delete_onclick }],
              'navigation': { 'prev': prev,
                              'next': next,
                              'up':  up + '#p-' + ph.basename },
              'page_title': ph.title,
              'photo': ph,
              'path': os.path.join(FORCE_SCRIPT_NAME, year, month, event),
              'prefix': FORCE_SCRIPT_NAME,
              'moderated': get_moderated_comments(),
              'user': get_user(request) }
    return render(request, 'photo.html', attrs)

@login_required
def info(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    return render(request, 'info.html',
                  { 'title': _('Information'),
                    'galeria': GALERIA_INFO,
                    'prefix': FORCE_SCRIPT_NAME,
                    'moderated': get_moderated_comments(),
                    'token': token.key })

@login_required
def password(request):
    if request.method == 'POST':
        message = response = ''
        password = request.POST.get('password', '')
        if len(password) > 0:
            request.user.set_password(password)
            request.user.save()
            response = 'success'
            message = _('Password updated.')
        else:
            response = 'failure'
            message = _('Invalid password.')
        return render(request, 'password.html',
                      { 'title': _('Change password'),
                        'galeria': GALERIA_INFO,
                        'moderated': get_moderated_comments(),
                        'response': response,
                        'message': message,
                        'prefix': FORCE_SCRIPT_NAME })
    else:
        return render(request, 'password.html',
                      { 'title': _('Change password'),
                        'galeria': GALERIA_INFO,
                        'moderated': get_moderated_comments(),
                        'prefix': FORCE_SCRIPT_NAME })

@login_required
def api(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    galeria_url = SERVER_URL + FORCE_SCRIPT_NAME
    return render(request, 'api.html',
                  { 'title': _('REST API'),
                    'galeria': GALERIA_INFO,
                    'galeria_url': galeria_url,
                    'prefix': FORCE_SCRIPT_NAME,
                    'moderated': get_moderated_comments(),
                    'token': token.key })

@login_required
def upload(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    if request.method == 'POST':
        form = UploadedImageForm(request.POST, request.FILES)
        files = request.FILES.getlist('files')
        if form.is_valid():
            c = 0
            for f in files:
                c += 1
                uploaded_image = UploadedImage(files=f)
                uploaded_image.save()
                Photo.add_photo(uploaded_image, token)
            n_files = len(files)
            if n_files > 0:
                message = (ngettext('The file was successfully uploaded.', \
                                    'A total of %(files)s files ' \
                                    'were successfully uploaded.', n_files) % \
                           { 'files': f'{n_files:n}' })
                return JsonResponse({ 'status': 'success',
                                      'message': message })
            return JsonResponse({ 'status': 'failure',
                                  'message': _('Invalid request.') })
        return JsonResponse({ 'status': 'failure',
                              'message': _('Something went wrong.') })
    else:
        form = UploadedImageForm()
        return render(request, 'upload.html',
                      { 'form': form,
                        'galeria': GALERIA_INFO,
                        'prefix': FORCE_SCRIPT_NAME,
                        'moderated': get_moderated_comments(),
                        'title': _('Upload photographs') })

@login_required
def map(request):
    path = request.GET.get('path', None)
    if not path:
        raise Http404('Missing path')
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = None
    t = path.split('/')
    if len(t) != 3:
        return redirect(request.META.get('HTTP_REFERER',
                                         FORCE_SCRIPT_NAME + '/'))
    year, month, event = t
    ya, ma, ea = Album.get_event(year, month, event)
    if not ya or not ma or not ea:
        raise Http404('The requested map does not exist')
    photos = ea.get_photos()
    markers = []
    for photo in photos:
        markers.append({ 'lab': photo.title,
                         'lat': photo.latitude,
                         'lng': photo.longitude })
    prev_url = next_url = None
    prev_event = ea.get_prev_url()
    if prev_event:
        prev_url = FORCE_SCRIPT_NAME + '/control/map?path=' + \
            prev_event.replace(FORCE_SCRIPT_NAME + '/', '')
    next_event = ea.get_next_url()
    if next_event:
        next_url = FORCE_SCRIPT_NAME + '/control/map?path=' + \
            next_event.replace(FORCE_SCRIPT_NAME + '/', '')

    n_photos = photos.count()
    page_title = _('Map of %(map)s') % { 'map': ea.name }
    summary = (ngettext('One photograph',
                        '%(photos)s photographs', n_photos) % \
               { 'photos': f'{n_photos:n}' })
    attrs = { 'breadcrumbs': [ { 'link': ya.get_url_path(),
                                 'name': ya.name },
                               { 'link': ma.get_url_path(),
                                 'name': ma.name },
                               { 'link': ea.get_url_path(),
                                 'name': ea.name },
                               { 'link': request.get_full_path(),
                                 'name': _('Mapa') } ],
              'prefix': FORCE_SCRIPT_NAME,
              'page_title': page_title,
              'galeria': GALERIA_INFO,
              'markers': markers,
              'summary': summary,
              'moderated': get_moderated_comments(),
              'navigation': { 'prev': prev_url,
                              'next': next_url,
                              'up': ea.get_url_path() } }
    return render(request, 'map.html', attrs)

def logout(request):
    django_logout(request)
    return redirect(FORCE_SCRIPT_NAME + '/')

def login(request):
    if request.method == 'POST':
        username = request.POST.get('username', '')
        password = request.POST.get('password', '')
        user = authenticate(request, username=username, password=password)
        next = request.POST.get('next', None)
        if user:
            django_login(request, user)
            if next:
                return redirect(next)
            token = Token.objects.get_or_create(user=request.user)[0]
            message = _('User %(user)s authenticated.') % { 'user': username }
            return render(request, 'info.html',
                          { 'title': 'Información',
                            'galeria': GALERIA_INFO,
                            'message': message,
                            'prefix': FORCE_SCRIPT_NAME,
                            'moderated': get_moderated_comments(),
                            'token': token.key,
                            'response': 'success' })
        return render(request, 'login.html',
                      { 'title': 'Entrar a la Galería',
                        'galeria': GALERIA_INFO,
                        'message': _('Invalid username or password.'),
                        'next': next,
                        'prefix': FORCE_SCRIPT_NAME,
                        'response': 'failure' })
    else:
        next = request.GET.get('next', '')
        if next == '':
            next = FORCE_SCRIPT_NAME
        return render(request, 'login.html',
                      { 'title': _('Login into Galería'),
                        'galeria': GALERIA_INFO,
                        'prefix': FORCE_SCRIPT_NAME,
                        'next': next })

@login_required
def reset(request):
    path = request.GET.get('path', '')
    if path == '/':
        Album.reset_all()
        return redirect(FORCE_SCRIPT_NAME + '/')
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = None
    t = path.split('/')
    if len(t) == 3:
        year, month, event = Album.get_event(*t)
        event.reset()
    elif len(t) == 2:
        year, month = Album.get_month(*t)
        month.reset()
    elif len(t) == 1:
        year = Album.get_year(t[0])
        year.reset()
    return redirect(FORCE_SCRIPT_NAME + '/' + path)

@login_required
def set_cover(request):
    path = request.GET.get('path', '')
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = photo = None
    t = path.split('/')
    if len(t) == 4:
        ya, ma, ea, ph = Photo.get_photo(*t)
        if not ya or not ma or not ea or not ph:
            raise Http404(os.path.join(year, month, event, photo) +
                          ' does not exist')
        ph.set_cover()
        return redirect(ph.get_url_path())
    elif len(t) == 3:
        ya, ma, ea = Album.get_event(*t)
        if not ya or not ma or not ea:
            raise Http404(os.path.join(year, month, event) + ' does not exist')
        ea.set_cover()
        return redirect(ea.get_url_path())
    elif len(t) == 2:
        ya, ma = Album.get_month(*t)
        if not ya or not ma:
            raise Http404(os.path.join(year, month) + ' does not exist')
        ma.set_cover()
        return redirect(ma.get_url_path())
    return redirect(FORCE_SCRIPT_NAME + '/')

@login_required
def private(request):
    path = request.GET.get('path', '')
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = photo = None
    t = path.split('/')
    token = Token.objects.get_or_create(user=request.user)[0]
    if len(t) == 4:
        ya, ma, ea, ph = Photo.get_photo(*t)
        if not ya or not ma or not ea or not ph:
            raise Http404(os.path.join(year, month, event, photo) +
                          ' does not exist')
        restrict(ph.make_private(), token)
        return redirect(ph.get_url_path())
    elif len(t) == 3:
        ya, ma, ea = Album.get_event(*t)
        if not ya or not ma or not ea:
            raise Http404(os.path.join(year, month, event) + ' does not exist')
        restrict(ea.make_private(), token)
        return redirect(ea.get_url_path())
    elif len(t) == 2:
        ya, ma = Album.get_month(*t)
        if not ya or not ma:
            raise Http404(os.path.join(year, month) + ' does not exist')
        restrict(ma.make_private(), token)
        return redirect(ma.get_url_path())
    elif len(t) == 1:
        ya = Album.get_year(t[0])
        if not ya:
            raise Http404(year + ' does not exist')
        restrict(ya.make_private(), token)
        return redirect(ya.get_url_path())
    return redirect(FORCE_SCRIPT_NAME + '/')

@login_required
def public(request):
    path = request.GET.get('path', '')
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = photo = None
    t = path.split('/')
    token = Token.objects.get_or_create(user=request.user)[0]
    if len(t) == 4:
        ya, ma, ea, ph = Photo.get_photo(*t)
        if not ya or not ma or not ea or not ph:
            raise Http404(os.path.join(year, month, event, photo) +
                          ' does not exist')
        free(ph.make_public(), token)
        return redirect(ph.get_url_path())
    elif len(t) == 3:
        ya, ma, ea = Album.get_event(*t)
        if not ya or not ma or not ea:
            raise Http404(os.path.join(year, month, event) + ' does not exist')
        free(ea.make_public(), token)
        return redirect(ea.get_url_path())
    elif len(t) == 2:
        ya, ma = Album.get_month(*t)
        if not ya or not ma:
            raise Http404(os.path.join(year, month) + ' does not exist')
        free(ma.make_public(), token)
        return redirect(ma.get_url_path())
    elif len(t) == 1:
        ya = Album.get_year(t[0])
        if not ya:
            raise Http404(year + ' does not exist')
        free(ya.make_public(), token)
        return redirect(ya.get_url_path())
    return redirect(FORCE_SCRIPT_NAME + '/')

@login_required
def download_covers(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    galeria_url = SERVER_URL + FORCE_SCRIPT_NAME
    return render(request, 'download-covers.html',
                  { 'title': _('Download covers'),
                    'galeria': GALERIA_INFO,
                    'galeria_url': galeria_url,
                    'token': token.key,
                    'moderated': get_moderated_comments(),
                    'prefix': FORCE_SCRIPT_NAME })

def _get_invalid_element(kind, name):
    message = _('Invalid %(kind)s: %(name)s') % { 'kind': kind, 'name': name }
    return { 'response': 'failure', 'message': message }

def _get_upload_covers_attrs(response, message, form):
    return { 'title': _('Upload covers'),
             'galeria': GALERIA_INFO,
             'form': form,
             'response': response,
             'message': message,
             'moderated': get_moderated_comments(),
             'prefix': FORCE_SCRIPT_NAME }

@login_required
def upload_covers(request):
    form = UploadedFileForm()
    if request.method == 'POST':
        form = UploadedFileForm(request.POST, request.FILES)
        file = request.FILES.get('file', '')
        if file != '' and form.is_valid():
            data = b''
            for chunk in file.chunks():
                data += chunk
            covers = json.loads(data)
            try:
                Album.set_covers(covers)
            except ValueError as ve:
                message = (_('Invalid %(kind)s: %(name)s') % \
                           { 'kind': ve[0], 'name': ve[1] })
                d = _get_upload_covers_attrs('failure', ve[1], form)
                return render(request, 'upload-covers.html', d)
            message = _('The covers were successfully uploaded.')
            d = _get_upload_covers_attrs('success', message, form)
            return render(request, 'upload-covers.html', d)
        message = _('Invalid file %(file)s.') % { 'file': file }
        d = _get_upload_covers_attrs('failure', message, form)
        return render(request, 'upload-covers.html', d)
    else:
        d = _get_upload_covers_attrs(None, None, form)
        return render(request, 'upload-covers.html', d)

@login_required
def comments(request):
    approve = int(request.GET.get('approve', '-1'))
    moderate = int(request.GET.get('moderate', '-1'))
    remove = int(request.GET.get('remove', '-1'))
    if approve != -1:
        comment = Comment.objects.get(id=approve)
        if not comment.is_public:
            comment.is_public = True
            comment.save()
    if moderate != -1:
        comment = Comment.objects.get(id=moderate)
        if comment.is_public:
            comment.is_public = False
            comment.save()
    if remove != -1:
        comment = Comment.objects.get(id=remove)
        comment.delete()
    moderated = get_moderated_comments()
    approved = get_approved_comments()
    tab = 'moderated' if 'tab' not in request.GET else request.GET['tab']
    return render(request, 'comments.html',
                  { 'title': _('Comments'),
                    'prefix': FORCE_SCRIPT_NAME,
                    'tab': tab,
                    'moderated': moderated,
                    'approved': approved })

@login_required
def delete(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    path = request.GET.get('path', '')
    redir = _delete(path, token)
    if redir:
        return redirect(redir)
    return redirect(FORCE_SCRIPT_NAME + '/')

@api_view(['GET'])
def list_albums(request):
    page = request.GET.get('page', 1)
    albums = Album.objects.filter(private=False).order_by('first_timestamp')
    paginator = Paginator(albums, LIST_ITEMS)
    try:
        the_page = paginator.page(page)
    except InvalidPage:
        return JsonResponse({})
    serializer = AlbumSerializer(the_page, many=True)
    return Response(serializer.data)

@api_view(['GET'])
def list_photos(request):
    page = request.GET.get('page', 1)
    photos = Photo.objects.filter(private=False).order_by('timestamp')
    paginator = Paginator(photos, LIST_ITEMS)
    try:
        the_page = paginator.page(page)
    except InvalidPage:
        return JsonResponse({})
    serializer = PhotoSerializer(the_page, many=True)
    return Response(serializer.data)

@api_view(['GET'])
def random_photo(request):
    photo = Album.get_random_photo(request.GET.get('blacklist', None))
    if not photo:
        return JsonResponse({ 'response': 'failure',
                              'message': _('The gallery is empty') })
    klass = 'gthumb' if photo.thumb_width > photo.thumb_height else 'gthumbv'
    image = SERVER_URL + FORCE_SCRIPT_NAME + photo.export_url
    link = SERVER_URL + photo.get_url_path()
    return JsonResponse({ 'response': 'success',
                          'class': klass,
                          'image': image,
                          'link': link,
                          'title': photo.title })

@api_view(['POST'])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def upload_photos(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    if request.method != 'POST':
        return JsonResponse({'response': 'failure',
                             'message': 'Invalid request'})
    private = request.GET.get('private', '') == 'true'
    form = UploadedImageSerializer(request.POST, request.FILES)
    files = request.FILES.getlist('files')
    if form.is_valid():
        for f in files:
            uploaded_image = UploadedImage(files=f)
            uploaded_image.save()
            Photo.add_photo(uploaded_image, token)
        if len(files) > 0:
            return JsonResponse({ 'response': 'success',
                                  'message': _('Upload successful') })
        else:
            return JsonResponse({ 'response': 'failure',
                                  'message': _('No files provided') })
    else:
        return JsonResponse({ 'response': 'failure',
                              'message': _('Error trying to upload photo') })

@api_view(['GET'])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def delete_item(request):
    token = Token.objects.get_or_create(user=request.user)[0]
    path = request.GET.get('path', '')
    if _delete(path, token):
        return JsonResponse({ 'response': 'success',
                              'message': _('Item deleted') })
    message = (_('Error deleting item: %(item)s') % { 'item': path })
    return JsonResponse({ 'response': 'fail', 'message': message })

@api_view(['GET'])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def reset_covers(request):
    r = {'response': 'success', 'message': _('Covers reset') }
    path = request.GET.get('path', '/')
    if path == '/':
        Album.reset_all()
        return JsonResponse(r)
    if path.endswith('/'):
        path = path[:-1]
    year = month = event = None
    t = path.split('/')
    if len(t) == 3:
        year, month, event = Album.get_event(*t)
        event.reset()
    elif len(t) == 2:
        year, month = Album.get_month(*t)
        month.reset()
    elif len(t) == 1:
        year = Album.get_year(t[0])
        year.reset()
    return JsonResponse(r)

@api_view(['GET'])
def get_covers(request):
    covers = Album.get_covers()
    return JsonResponse(covers)

@api_view(['POST'])
@parser_classes([JSONParser])
@authentication_classes([TokenAuthentication])
@permission_classes([IsAuthenticated])
def set_covers(request):
    if request.method != 'POST':
        return JsonResponse({'response': 'failure',
                             'message': _('Invalid request') })
    try:
        Album.set_covers(request.data)
    except ValueError as ve:
        message = (_('Invalid %(kind)s: %(name)s') % \
                   { 'kind': ve[0], 'name': ve[1] })
        return JsonResponse({'response': 'failure', 'message': message })
    message = _('The covers were successfully uploaded.')
    return JsonResponse({ 'response': 'success', 'message': message })

def handler404(request, exception):
    attrs = { 'breadcrumbs': [],
              'galeria': GALERIA_INFO,
              'link': FORCE_SCRIPT_NAME + '/',
              'message': _('The requested page does not exists.'),
              'navigation': { 'prev': None, 'next': None,
                              'up': FORCE_SCRIPT_NAME + '/' },
              'page_title': _('Page not found'),
              'path': FORCE_SCRIPT_NAME,
              'prefix': FORCE_SCRIPT_NAME,
              'summary': _('Check the URL') }
    return render(request, 'not-found.html', attrs)
